/* OpenVAS-Client
 * $Id$
 * Description: The ssh key management dialog.
 *
 * Authors:
 * Felix Wolfsteller <felix.wolfsteller@intevation.de>
 *
 * Copyright:
 * Copyright (C) 2008 Intevation GmbH
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * or, at your option, any later version as published by the Free
 * Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * In addition, as a special exception, you have
 * permission to link the code of this program with the OpenSSL
 * library (or with modified versions of OpenSSL that use the same
 * license as OpenSSL), and distribute linked combinations including
 * the two. You must obey the GNU General Public License in all
 * respects for all of the code used other than OpenSSL. If you
 * modify this file, you may extend this exception to your version
 * of the file, but you are not obligated to do so. If you do not
 * wish to do so, delete this exception statement from your version.
 */

#include <includes.h>
#include "nessus_i18n.h"

#ifdef USE_GTK
#include <gtk/gtk.h>
#include "error_dlg.h"
#include "listnotebook.h"
#include "openvas_ssh_login.h"
#include "preferences.h"
#include "prefs_dialog_plugins_prefs.h"
#include "ssh_keys_dialog.h"
#include "ssh_key_info_form.h"


/**
 * Callback function to open the OpenVAS SSH Key Manager.
 * Only calls ssh_keys_dialog_show.
 * @see ssh_keys_dialog_show
 */
void ssh_manager_button_cb(GtkWidget* super, void* ignored )
{
  // So far no pre- or post conditions, show dialog directly.
  ssh_keys_dialog_show();
}


/**
 * Creates a ssh_key_info_form from an openvas_ssh_login and adds it as a page to
 * a Listnotebook.
 * @param _ignored --ignored-- (might be non-NULL if used as callback).
 * @param login Login to create a form from.
 * @param listnotebook Listnotebook to add the page to.
 */
static void
ssh_keys_dialog_add_key(char* _ignored, openvas_ssh_login* login,
                        GtkWidget* listnotebook)
{
  ssh_key_info_form* form = ssh_key_info_form_create();
  ssh_key_info_form_fill(form, login);
  listnotebook_add_page(listnotebook, form->box, login->name, NULL);
  gtk_widget_show_all(listnotebook);
}


/**
 * Prompts the user for information about a new ssh login.
 * (Shows a dialog with an editable ssh_key_info_form).
 * The listnotebook parameter is passed on to the dialog.
 * If creation of a ssh keypair was successfull, information about the
 * openvas_ssh_login will be added to the notebook.
 * @param parent --ignored-- (might be non-NULL if used as callback).
 * @param notebook Listnotebook to add a page to if ssh keypair generation was
 *                 successfull.
 */
static void
ssh_keys_dialog_prompt_new(GtkWidget* parent, GtkWidget* notebook)
{
  GtkWidget* dialog;
  GtkWidget* content_area;
  ssh_key_info_form* form;
  form = ssh_key_info_form_create();
  
  dialog = gtk_dialog_new_with_buttons(_("OpenVAS SSH Key Manager"),
                                    NULL,
                                    GTK_DIALOG_MODAL
                                        | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_STOCK_OK,
                                    GTK_RESPONSE_OK,
                                    GTK_STOCK_CANCEL,
                                    GTK_RESPONSE_CANCEL,
                                    NULL);
  
  content_area = GTK_DIALOG(dialog)->vbox;
  gtk_container_add (GTK_CONTAINER (content_area), form->box );
  
  gtk_window_set_default_size(GTK_WINDOW(dialog), 640, 240);
  gtk_widget_show_all(dialog);
  
  gint result = gtk_dialog_run (GTK_DIALOG (dialog));
  while (result == GTK_RESPONSE_OK && ssh_key_info_form_validated(form) == FALSE )
    result = gtk_dialog_run(GTK_DIALOG(dialog));
  switch (result)
  {
    case GTK_RESPONSE_OK:
      {
      // Check entries
      openvas_ssh_login* loginfo = ssh_key_info_form_produce_login(form);
      if( openvas_ssh_login_create(loginfo) == FALSE )
      {
        //Error message have been shown by openvas_ssh_login_create function
        break;
      }
  
      if(Global->sshkeys == NULL)
        Global->sshkeys = g_hash_table_new_full(g_str_hash, g_str_equal, NULL,
          (GDestroyNotify) openvas_ssh_login_free);
      g_hash_table_insert(Global->sshkeys, loginfo->name, loginfo); 
      
      char* loginsfile = g_build_filename(prefs_get_nessushome(), ".openvas", ".ssh", ".logins", NULL);
      openvas_ssh_login_file_write(Global->sshkeys, loginsfile);
      efree(&loginsfile);
      
      ssh_keys_dialog_add_key(NULL, loginfo, notebook);
      prefs_dialog_plugins_prefs_update_ssh_comboboxes(Global, loginfo->name);
      break;
      }
    default:
      // do nothing since dialog was cancelled or closed
      break;
  }
   gtk_widget_destroy(dialog);
}


/**
 * Shows the SSH Key Management Dialog.
 */
void
ssh_keys_dialog_show()
{
  GtkWidget* win;
  GtkWidget* separator;
  GtkWidget* content_area;
  GtkWidget* key_notebook;

  win = gtk_dialog_new_with_buttons(_("OpenVAS SSH Key Manager"),
                                    NULL,
                                    GTK_DIALOG_MODAL
                                        | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_STOCK_CLOSE,
                                    GTK_RESPONSE_ACCEPT,
                                    NULL);
  g_signal_connect_swapped (win,
                            "response", 
                            G_CALLBACK (gtk_widget_destroy),
                            win);
  
  content_area = GTK_DIALOG(win)->vbox;
  
  key_notebook = listnotebook_new(TRUE, TRUE);
  
  // For each key, add a notebookpage
  if(Global->sshkeys)
    g_hash_table_foreach(Global->sshkeys, (GHFunc) ssh_keys_dialog_add_key, 
                         key_notebook);
  
  // Add listnotebook with keys
  gtk_container_add (GTK_CONTAINER (content_area), key_notebook);
  gtk_widget_show(key_notebook);
  
  // Add a separator
  separator = gtk_hseparator_new();
  gtk_container_add (GTK_CONTAINER (content_area), separator);

  /* Add action buttons */
  GtkWidget* button_new = gtk_button_new_with_label(_("Create a new keypair"));
  gtk_container_add (GTK_CONTAINER (content_area), button_new);
  g_signal_connect(button_new, "clicked", (GtkSignalFunc) ssh_keys_dialog_prompt_new, key_notebook);
  
  gtk_window_set_default_size(GTK_WINDOW(win), 640, 240);
  gtk_widget_show_all(win);
}
#endif /* USE_GTK */
